# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    HprimList,
    HprimMultiple,
    Segment,
)

logger = logging.getLogger("driverng.cellmek")


class CellmekSegmentH(HeaderSegment):
    sender_name = HprimString((5, 1))
    device_id = HprimString((5, 2))
    version = HprimString(13, default="LIS2-A2")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)
    infos = HprimForeign("M", "header")


class CellmekSegmentP(Segment):
    laboratory_patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    attending_physician_id = HprimString((14, 1))
    attending_physician_last_name = HprimString((14, 2))
    attending_physician_first_name = HprimString((14, 3))
    admission_status = HprimString(25)
    patient_location = HprimString(26)

    orders = HprimForeign("O", "patient")


class CellmekSegmentO(Segment):
    specimen_id = HprimString(3)
    tests = HprimList(5, basefield=HprimMultiple(0, code=HprimString(4)))
    priority = HprimString(6)
    collection_date = HprimDate(8)
    action_code = HprimString(12)
    specimen_type = HprimString(16)
    ordering_physician_id = HprimString((17, 1))
    ordering_physician_last_name = HprimString((17, 2))
    ordering_physician_first_name = HprimString((17, 3))
    report_type = HprimString(26, default="O")

    infos = HprimForeign("M", "order")


class CellmekSegmentC(Segment):
    source = HprimString(3)
    text = HprimString(4)
    type = HprimString(5, default="G")


class CellmekSegmentQ(Segment):
    patient_id = HprimString((3, 1))
    specimen_id = HprimString((3, 2))
    request_status = HprimString(13)


class CellmekSegmentL(Segment):
    termination_code = HprimString(3, default="N")


class CellmekSegmentM(Segment):
    test_id = HprimString(3)
    test_date = HprimDate(4)
    version = HprimString(5, default="LIS2-A2")


class Driver(AstmMessageV2):
    SPEC = {"code": "cellmek", "label": "Beckman Cellmek"}
    SEGMENT_CLASS_MAP = {
        "H": CellmekSegmentH,
        "P": CellmekSegmentP,
        "O": CellmekSegmentO,
        "C": CellmekSegmentC,
        "Q": CellmekSegmentQ,
        "L": CellmekSegmentL,
        "M": CellmekSegmentM,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    ENCODING = "UTF-8"

    OPTIONS = [
        AstmMessageV2.Option(
            "cellmek_sender_name",
            label="Paramétrer le nom de l'émetteur du segment H.5.1",
            default="MCA",
        )
    ]

    def __init__(self, **kwargs):
        self.cellmek_sender_name = kwargs.pop("cellmek_sender_name", "MCA")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = CellmekSegmentH(type_segment="H", **kwargs)
        message.end = CellmekSegmentL(type_segment="L")
        return message

    def get_physician(self, patient, quality):
        for p in patient.physicians:
            if p.type.lower() == quality:
                return p.code, p.last_name, p.first_name
        return None, None, None

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"], sender_name=self.cellmek_sender_name)
        if not msg.sender_name:
            msg.sender_name = worklist.sender_id
        worklist = worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", worklist.ELSE: "A"},
            priority={"S": "R", "A": "R", worklist.ELSE: "R"},
        )

        list_good_sample = []
        for patient in worklist:
            attending_physician_id, attending_physician_last_name, attending_physician_first_name = self.get_physician(
                patient, "attending"
            )
            ordering_physician_id, ordering_physician_last_name, ordering_physician_first_name = self.get_physician(
                patient, "ordering"
            )

            # On interroge des specimen_id, worklist nous les retourne dans les patient_id si elle les trouve
            if patient.patient_id not in list_good_sample:
                list_good_sample.append("{}".format(patient.patient_id))

            seg_p = msg.patients.add(
                laboratory_patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                attending_physician_id=attending_physician_id,
                attending_physician_last_name=attending_physician_last_name,
                attending_physician_first_name=attending_physician_first_name,
                admission_status=patient.admission_status,
                patient_location=patient.location,
            )
            for sample, action, priority in patient.samples:
                for i, order in enumerate(sample.orders, start=1):
                    if i % 3 == 1:
                        o = {
                            "specimen_id": sample.sample_id,
                            "priority": priority,
                            "collection_date": sample.collection_date,
                            "action_code": action,
                            "specimen_type": sample.specimen,
                            "ordering_physician_id": ordering_physician_id,
                            "ordering_physician_last_name": ordering_physician_last_name,
                            "ordering_physician_first_name": ordering_physician_first_name,
                            "report_type": "O" if sample.orders else "Y",
                        }
                        seg_o = seg_p.orders.add(**o)
                    t = seg_o.tests.add()
                    t.code = order.code
                if not sample.orders:
                    o = {
                        "specimen_id": sample.sample_id,
                        "priority": priority,
                        "collection_date": sample.collection_date,
                        "action_code": action,
                        "specimen_type": sample.specimen,
                        "ordering_physician_id": ordering_physician_id,
                        "ordering_physician_last_name": ordering_physician_last_name,
                        "ordering_physician_first_name": ordering_physician_first_name,
                        "report_type": "O" if sample.orders else "Y",
                    }
                    seg_o = seg_p.orders.append(**o)

        if context["query"]:
            # on regarde les query specimen_id auquel on n'a pas de patient_id
            if len(context["query"].queries) != len(msg.patients):
                for sample in context["query"].queries:
                    sample_id = sample.specimen_id

                    # On interroge des specimen_id, worklist nous les retourne dans les patient_id si elle les trouve
                    if sample_id in list_good_sample:
                        continue
                    list_good_sample.append("{}".format(sample_id))

                    seg_p = msg.patients.add()
                    seg_o = seg_p.orders.add(specimen_id=sample_id, report_type="Y")

        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.specimen_id)
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample_id = seg_o.specimen_id
                comments = []
                if seg_o.commentaires:
                    for seg_c in seg_o.commentaires:
                        comments.append(seg_c.text)
                    sample = worklist.get_or_create(sample_id=sample_id, type="patient")

                    sample.results.add(
                        code="REJET", value=seg_o.report_type, comments=comments
                    )

        return worklist
