# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "cobas_pvt"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||MPL|||||PVT|||P|
    P|1|036666666||123456|NYME^ANO||20110513|F|||||||||||||||20130603110342|
    O|1|123456|0|^^^A^0\\^^^URG^0|S|20130603110342|||||N|
    O|2|123456|0|^^^B^0|S|20130603110342|||||A|
    O|3|123456|0|^^^C^0|S|20130603110342|||||C|
    L|1|N|"""
    assert message == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
        H|\\^&|||PVT^RSA^ 5.60/23.09.2004|||||HOST||P
        Q|1|^0014040953^0^0^^5||ALL||||||||O
        L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&||||MPL|||||PVT|||P|
    P|1|0014040953|0014040953|||||||||||||||||||||
    O|1|0014040953|0||R||||||N||||||||X||||||Z
    L|1|N|"""
    assert result == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||PVT^RSA^ 5.60/23.09.2004|||||HOST||P
    Q|1|^0014040953^0^0^^5||ALL||||||||O
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "0014040953", "type": "patient"}],
    }
    assert worklist == expected


def test_decode_information_sorting_pvt(self):
    worklist = self.decode(
        """
    H|\\^&|||PVT^RSA^ 6.0 /25.03.2008|||||HOST||P
    M|1|IS|4050160596|2|0^0|0|2173|193|3218|0|1250|1010|1|
    M|2|IS|4050160694|2|1^0|0|2402|75|3386|0|1250|1010|1|
    M|3|IS|4050160671|2|2^0|0|2389|226|3520|0|1250|1010|2|
    L|1|N
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected


def test_decode_sorting_list_pvt(self):
    worklist = self.decode(
        """
    H|\\^&|||PVT^RSA^ 6.0 /25.03.2008|||||HOST||P
    M|1|LI|ALQT_RSA_S|1^4|AL529598M01|ALQT_RSA|0920|20080925212711|XA|AL529598M03||
    M|2|LI|ALQT_RSA_S|1^5|AL529593M03|ALQT_RSA|0600|20080925212719|XA|AL529593M05||
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "ALQT_RSA_S-1-4",
                "sample_id": "AL529598M01",
                "type": "patient",
                "results": [
                    {
                        "code": "TRACKINGP612",
                        "comments": ["Volume 0920 microlitres"],
                        "date": datetime.datetime(2008, 9, 25, 21, 27, 11),
                        "instrument": "ALQT_RSA_S",
                        "value": "ALQT_RSA_S",
                    }
                ],
            },
            {
                "rack": "ALQT_RSA_S-1-5",
                "sample_id": "AL529593M03",
                "type": "patient",
                "results": [
                    {
                        "code": "TRACKINGP612",
                        "comments": ["Volume 0600 microlitres"],
                        "date": datetime.datetime(2008, 9, 25, 21, 27, 19),
                        "instrument": "ALQT_RSA_S",
                        "value": "ALQT_RSA_S",
                    }
                ],
            },
        ],
    }
    assert worklist == expected


def test_tray_change_pvt(self):
    worklist = self.decode(
        """
    H|\\^&|||PVT^RSA^ 6.0 /25.03.2008|||||HOST||P
    M|1|TR|PT_STORAGE|1|20080925212010|
    L|1|F
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected


def test_result_pvt_rack(self):
    worklist = self.decode(
        """
    H|\\^&|||PVT^RSA^ 7.0 /yyyy-mm-dd|||||HOST||P
    M|1|LI|ALICOM|5^3|890112099827|ALICOM|0300|20190912142708|XP||
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "ALICOM-5-3",
                "sample_id": "890112099827",
                "type": "patient",
                "results": [
                    {
                        "code": "TRACKINGP612",
                        "comments": ["Volume 0300 microlitres"],
                        "date": datetime.datetime(2019, 9, 12, 14, 27, 8),
                        "instrument": "ALICOM",
                        "value": "ALICOM",
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_result_pvt_serotheque(self):
    worklist = self.decode(
        """
    H|\\^&|||PVT^RSA^ 7.0 /yyyy-mm-dd|||||HOST||P
    M|1|LI|SEROT_1|13^1|810121040557|TSERO|0000|20210421190431|XP||
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "SEROT_1-13-1",
                "results": [
                    {
                        "code": "TRACKINGP612",
                        "comments": ["Volume 0000 microlitres"],
                        "date": datetime.datetime(2021, 4, 21, 19, 4, 31),
                        "instrument": "SEROT_1",
                        "value": "SEROT_1",
                    },
                    {"code": "P612RACK", "value": "SEROT_1-1"},
                ],
                "sample_id": "810121040557",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_result_pvt_rack_archive(self):
    worklist = self.decode(
        """
    H|\\^&|||PVT^RSA^ 7.0 /yyyy-mm-dd|||||HOST||P
    M|1|LI|NARCHIVE^29|1^41|890112099801|TARCHI|0000|20190912145606|XP||
    M|2|LI|NARCHIVE^29|1^42|890112099810|TARCHI|0000|20190912145610|XP||
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "29-1-41",
                "sample_id": "890112099801",
                "type": "patient",
                "results": [
                    {
                        "code": "TRACKINGP612",
                        "comments": ["Volume 0000 microlitres"],
                        "date": datetime.datetime(2019, 9, 12, 14, 56, 6),
                        "instrument": "NARCHIVE",
                        "value": "NARCHIVE",
                    }
                ],
            },
            {
                "rack": "29-1-42",
                "sample_id": "890112099810",
                "type": "patient",
                "results": [
                    {
                        "code": "TRACKINGP612",
                        "comments": ["Volume 0000 microlitres"],
                        "date": datetime.datetime(2019, 9, 12, 14, 56, 10),
                        "instrument": "NARCHIVE",
                        "value": "NARCHIVE",
                    }
                ],
            },
        ],
    }
    assert worklist == expected
